# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..commons.types.entity_id_base import EntityIdBase
from ..commons.types.action_parameter import ActionParameter
from ..commons.types.precondition import Precondition
from ..core.request_options import RequestOptions
from ..commons.types.action_response import ActionResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.pydantic_utilities import parse_obj_as
from ..commons.errors.not_found_error import NotFoundError
from ..commons.types.error_message import ErrorMessage
from ..commons.errors.bad_request_error import BadRequestError
from ..commons.errors.server_error import ServerError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.jsonable_encoder import jsonable_encoder
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ActionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create_or_update(
        self,
        *,
        action_id: EntityIdBase,
        name: str,
        description: str,
        user_interaction_required: bool,
        user_form_parameters: typing.Sequence[ActionParameter],
        button_name: typing.Optional[str] = OMIT,
        precondition: typing.Optional[Precondition] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ActionResponse:
        """
        Update an action or create it if it doesn't exist

        Parameters
        ----------
        action_id : EntityIdBase
            ID that uniquely identifies this action

        name : str
            The name of the action. This is displayed to the end user as part of forms when user interaction is required. It is also used to help Maven decide if the action is relevant to a conversation.

        description : str
            The description of the action. Must be less than 1024 characters. This helps Maven decide if the action is relevant to a conversation and is not displayed directly to the end user. Descriptions are used by the LLM.

        user_interaction_required : bool
            Whether the action requires user interaction to execute. If false, and all of the required action parameters are known, the LLM may call the action automatically. If true, an conversations ask call will return a BotActionFormResponse which must be submitted by an API caller. API callers must display a button with the buttonName label to confirm the user's intent.

        user_form_parameters : typing.Sequence[ActionParameter]
            The parameters that the action uses as input. An action will only be executed when all of the required parameters are provided. During execution, actions all have access to the full Conversation and User objects. Parameter values may be inferred from the user's conversation by the LLM.

        button_name : typing.Optional[str]
            When user interaction is required, the name of the button that is shown to the end user to confirm execution of the action

        precondition : typing.Optional[Precondition]
            The preconditions that must be met for an action to be relevant to a conversation. Can be used to restrict actions to certain types of users.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ActionResponse

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.commons import EntityIdBase, Precondition_Group, Precondition_User

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.actions.create_or_update(
            action_id=EntityIdBase(
                reference_id="get-balance",
            ),
            name="Get the user's balance",
            description="This action calls an API to get the user's current balance.",
            user_interaction_required=False,
            user_form_parameters=[],
            precondition=Precondition_Group(
                operator="AND",
                preconditions=[
                    Precondition_User(
                        key="userKey",
                    ),
                    Precondition_User(
                        key="userKey2",
                    ),
                ],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/actions",
            method="PUT",
            json={
                "actionId": convert_and_respect_annotation_metadata(
                    object_=action_id, annotation=EntityIdBase, direction="write"
                ),
                "name": name,
                "description": description,
                "userInteractionRequired": user_interaction_required,
                "buttonName": button_name,
                "precondition": convert_and_respect_annotation_metadata(
                    object_=precondition, annotation=Precondition, direction="write"
                ),
                "userFormParameters": convert_and_respect_annotation_metadata(
                    object_=user_form_parameters, annotation=typing.Sequence[ActionParameter], direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ActionResponse,
                    parse_obj_as(
                        type_=ActionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, action_reference_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ActionResponse:
        """
        Get an action by its supplied ID

        Parameters
        ----------
        action_reference_id : str
            The reference ID of the action to get. All other entity ID fields are inferred from the request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ActionResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.actions.get(
            action_reference_id="get-balance",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/actions/{jsonable_encoder(action_reference_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ActionResponse,
                    parse_obj_as(
                        type_=ActionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, action_reference_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete an action

        Parameters
        ----------
        action_reference_id : str
            The reference ID of the action to unregister. All other entity ID fields are inferred from the request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.actions.delete(
            action_reference_id="get-balance",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/actions/{jsonable_encoder(action_reference_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncActionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create_or_update(
        self,
        *,
        action_id: EntityIdBase,
        name: str,
        description: str,
        user_interaction_required: bool,
        user_form_parameters: typing.Sequence[ActionParameter],
        button_name: typing.Optional[str] = OMIT,
        precondition: typing.Optional[Precondition] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ActionResponse:
        """
        Update an action or create it if it doesn't exist

        Parameters
        ----------
        action_id : EntityIdBase
            ID that uniquely identifies this action

        name : str
            The name of the action. This is displayed to the end user as part of forms when user interaction is required. It is also used to help Maven decide if the action is relevant to a conversation.

        description : str
            The description of the action. Must be less than 1024 characters. This helps Maven decide if the action is relevant to a conversation and is not displayed directly to the end user. Descriptions are used by the LLM.

        user_interaction_required : bool
            Whether the action requires user interaction to execute. If false, and all of the required action parameters are known, the LLM may call the action automatically. If true, an conversations ask call will return a BotActionFormResponse which must be submitted by an API caller. API callers must display a button with the buttonName label to confirm the user's intent.

        user_form_parameters : typing.Sequence[ActionParameter]
            The parameters that the action uses as input. An action will only be executed when all of the required parameters are provided. During execution, actions all have access to the full Conversation and User objects. Parameter values may be inferred from the user's conversation by the LLM.

        button_name : typing.Optional[str]
            When user interaction is required, the name of the button that is shown to the end user to confirm execution of the action

        precondition : typing.Optional[Precondition]
            The preconditions that must be met for an action to be relevant to a conversation. Can be used to restrict actions to certain types of users.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ActionResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.commons import EntityIdBase, Precondition_Group, Precondition_User

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.actions.create_or_update(
                action_id=EntityIdBase(
                    reference_id="get-balance",
                ),
                name="Get the user's balance",
                description="This action calls an API to get the user's current balance.",
                user_interaction_required=False,
                user_form_parameters=[],
                precondition=Precondition_Group(
                    operator="AND",
                    preconditions=[
                        Precondition_User(
                            key="userKey",
                        ),
                        Precondition_User(
                            key="userKey2",
                        ),
                    ],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/actions",
            method="PUT",
            json={
                "actionId": convert_and_respect_annotation_metadata(
                    object_=action_id, annotation=EntityIdBase, direction="write"
                ),
                "name": name,
                "description": description,
                "userInteractionRequired": user_interaction_required,
                "buttonName": button_name,
                "precondition": convert_and_respect_annotation_metadata(
                    object_=precondition, annotation=Precondition, direction="write"
                ),
                "userFormParameters": convert_and_respect_annotation_metadata(
                    object_=user_form_parameters, annotation=typing.Sequence[ActionParameter], direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ActionResponse,
                    parse_obj_as(
                        type_=ActionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, action_reference_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ActionResponse:
        """
        Get an action by its supplied ID

        Parameters
        ----------
        action_reference_id : str
            The reference ID of the action to get. All other entity ID fields are inferred from the request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ActionResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.actions.get(
                action_reference_id="get-balance",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/actions/{jsonable_encoder(action_reference_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ActionResponse,
                    parse_obj_as(
                        type_=ActionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, action_reference_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete an action

        Parameters
        ----------
        action_reference_id : str
            The reference ID of the action to unregister. All other entity ID fields are inferred from the request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.actions.delete(
                action_reference_id="get-balance",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/actions/{jsonable_encoder(action_reference_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
from ...commons.types.precondition_group import PreconditionGroup
import pydantic
from .knowledge_base_type import KnowledgeBaseType
import typing
from ...commons.types.precondition import Precondition
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.pydantic_utilities import update_forward_refs


class KnowledgeBaseProperties(UniversalBaseModel):
    name: str = pydantic.Field()
    """
    The name of the knowledge base
    """

    type: KnowledgeBaseType = pydantic.Field()
    """
    The type of the knowledge base. Can not be changed once created.
    """

    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The URL to pull content from for RSS and URL knowledge bases.
    """

    precondition: typing.Optional[Precondition] = pydantic.Field(default=None)
    """
    (Beta) The preconditions that must be met for knowledge base be relevant to a conversation. Can be used to limit knowledge to certain types of users.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


update_forward_refs(PreconditionGroup, KnowledgeBaseProperties=KnowledgeBaseProperties)



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing_extensions
from ...core.serialization import FieldMetadata
import typing
from .action_form_field import ActionFormField
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class BotActionFormResponse(UniversalBaseModel):
    """
    This response should be rendered as a form which users can submit. Upon submission call the submit action form API.
    """

    id: str
    form_label: typing_extensions.Annotated[str, FieldMetadata(alias="formLabel")]
    fields: typing.List[ActionFormField]
    submit_label: typing_extensions.Annotated[str, FieldMetadata(alias="submitLabel")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing_extensions
import typing
from ...core.serialization import FieldMetadata
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
from ...commons.types.action_form_field import ActionFormField
from ...commons.types.chart_spec_schema import ChartSpecSchema
from ...commons.types.source import Source
from ...commons.types.entity_id import EntityId
from ...commons.types.error_message import ErrorMessage


class StreamResponse_Text(UniversalBaseModel):
    event_type: typing_extensions.Annotated[typing.Literal["text"], FieldMetadata(alias="eventType")] = "text"
    contents: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class StreamResponse_Action(UniversalBaseModel):
    event_type: typing_extensions.Annotated[typing.Literal["action"], FieldMetadata(alias="eventType")] = "action"
    id: str
    form_label: typing_extensions.Annotated[str, FieldMetadata(alias="formLabel")]
    fields: typing.List[ActionFormField]
    submit_label: typing_extensions.Annotated[str, FieldMetadata(alias="submitLabel")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class StreamResponse_Chart(UniversalBaseModel):
    event_type: typing_extensions.Annotated[typing.Literal["chart"], FieldMetadata(alias="eventType")] = "chart"
    label: str
    spec_schema: typing_extensions.Annotated[ChartSpecSchema, FieldMetadata(alias="specSchema")]
    spec: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class StreamResponse_Metadata(UniversalBaseModel):
    event_type: typing_extensions.Annotated[typing.Literal["metadata"], FieldMetadata(alias="eventType")] = "metadata"
    followup_questions: typing_extensions.Annotated[typing.List[str], FieldMetadata(alias="followupQuestions")]
    sources: typing.List[Source]
    language: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class StreamResponse_Start(UniversalBaseModel):
    event_type: typing_extensions.Annotated[typing.Literal["start"], FieldMetadata(alias="eventType")] = "start"
    conversation_message_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="conversationMessageId")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class StreamResponse_End(UniversalBaseModel):
    event_type: typing_extensions.Annotated[typing.Literal["end"], FieldMetadata(alias="eventType")] = "end"
    error: typing.Optional[ErrorMessage] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


StreamResponse = typing.Union[
    StreamResponse_Text,
    StreamResponse_Action,
    StreamResponse_Chart,
    StreamResponse_Metadata,
    StreamResponse_Start,
    StreamResponse_End,
]



# This file was auto-generated by Fern from our API Definition.

from .feedback_base import FeedbackBase
import typing_extensions
from .entity_id import EntityId
from ...core.serialization import FieldMetadata
import pydantic
import typing
import datetime as dt
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class Feedback(FeedbackBase):
    """
    Examples
    --------
    from mavenagi.commons import EntityId, Feedback

    Feedback(
        feedback_id=EntityId(
            reference_id="feedback-0",
            app_id="myapp",
            organization_id="acme",
            agent_id="support",
            type="FEEDBACK",
        ),
        conversation_id=EntityId(
            reference_id="conversation-0",
            app_id="myapp",
            organization_id="acme",
            agent_id="support",
            type="CONVERSATION",
        ),
        conversation_message_id=EntityId(
            reference_id="message-1",
            app_id="myapp",
            organization_id="acme",
            agent_id="support",
            type="CONVERSATION_MESSAGE",
        ),
        user_id=EntityId(
            reference_id="user-0",
            app_id="myapp",
            organization_id="acme",
            agent_id="support",
            type="USER",
        ),
        type="THUMBS_UP",
        text="Great answer!",
    )
    """

    feedback_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="feedbackId")] = pydantic.Field()
    """
    The ID of the piece of feedback
    """

    conversation_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="conversationId")] = pydantic.Field()
    """
    The ID of the conversation the feedback is about
    """

    conversation_message_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="conversationMessageId")] = (
        pydantic.Field()
    )
    """
    The ID of the conversation message the feedback is about
    """

    user_id: typing_extensions.Annotated[typing.Optional[EntityId], FieldMetadata(alias="userId")] = pydantic.Field(
        default=None
    )
    """
    The ID of the user who created the feedback
    """

    created_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="createdAt")] = (
        pydantic.Field(default=None)
    )
    """
    The date and time the feedback was created
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing_extensions
import typing
import datetime as dt
from ...core.serialization import FieldMetadata
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ConversationMessageBase(UniversalBaseModel):
    created_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="createdAt")] = (
        pydantic.Field(default=None)
    )
    """
    The date and time the conversation was created
    """

    updated_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="updatedAt")] = (
        pydantic.Field(default=None)
    )
    """
    The date and time the conversation was last updated
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .base_knowledge_document import BaseKnowledgeDocument
import typing_extensions
from ...commons.types.entity_id_base import EntityIdBase
from ...core.serialization import FieldMetadata
import pydantic
from .knowledge_document_content_type import KnowledgeDocumentContentType
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class KnowledgeDocumentRequest(BaseKnowledgeDocument):
    """
    Examples
    --------
    from mavenagi.commons import EntityIdBase
    from mavenagi.knowledge import KnowledgeDocumentRequest

    KnowledgeDocumentRequest(
        knowledge_document_id=EntityIdBase(
            reference_id="getting-started",
        ),
        content_type="MARKDOWN",
        content="## Getting started\\nThis is a getting started guide for the help center.",
        title="Getting started",
    )
    """

    knowledge_document_id: typing_extensions.Annotated[EntityIdBase, FieldMetadata(alias="knowledgeDocumentId")] = (
        pydantic.Field()
    )
    """
    ID that uniquely identifies this knowledge document within its knowledge base
    """

    content_type: typing_extensions.Annotated[KnowledgeDocumentContentType, FieldMetadata(alias="contentType")]
    content: str = pydantic.Field()
    """
    The content of the document. Not shown directly to users. May be provided in HTML or markdown. HTML will be converted to markdown automatically. Images are not currently supported and will be ignored.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing
import typing_extensions
from .entity_id import EntityId
from ...core.serialization import FieldMetadata
from .user_message_attachment import UserMessageAttachment
from .entity_id_base import EntityIdBase
from .user_conversation_message_type import UserConversationMessageType
import datetime as dt
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
from .bot_conversation_message_type import BotConversationMessageType
from .bot_response import BotResponse
from .bot_response_metadata import BotResponseMetadata


class ConversationMessageResponse_User(UniversalBaseModel):
    type: typing.Literal["user"] = "user"
    conversation_message_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="conversationMessageId")]
    language: typing.Optional[str] = None
    attachments: typing.List[UserMessageAttachment]
    user_id: typing_extensions.Annotated[EntityIdBase, FieldMetadata(alias="userId")]
    text: str
    user_message_type: typing_extensions.Annotated[UserConversationMessageType, FieldMetadata(alias="userMessageType")]
    created_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="createdAt")] = None
    updated_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="updatedAt")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ConversationMessageResponse_Bot(UniversalBaseModel):
    type: typing.Literal["bot"] = "bot"
    conversation_message_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="conversationMessageId")]
    bot_message_type: typing_extensions.Annotated[BotConversationMessageType, FieldMetadata(alias="botMessageType")]
    responses: typing.List[BotResponse]
    metadata: BotResponseMetadata
    created_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="createdAt")] = None
    updated_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="updatedAt")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


ConversationMessageResponse = typing.Union[ConversationMessageResponse_User, ConversationMessageResponse_Bot]



# This file was auto-generated by Fern from our API Definition.

from .app_user import AppUser
import typing_extensions
from .entity_id_base import EntityIdBase
from ...core.serialization import FieldMetadata
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class AppUserRequest(AppUser):
    """
    Examples
    --------
    from mavenagi.commons import (
        AppUserIdentifier,
        AppUserRequest,
        EntityIdBase,
        UserData,
    )

    AppUserRequest(
        user_id=EntityIdBase(
            reference_id="user-0",
        ),
        identifiers=[
            AppUserIdentifier(
                value="joe@myapp.com",
                type="EMAIL",
            )
        ],
        data={
            "name": UserData(
                value="Joe",
                visibility="VISIBLE",
            )
        },
    )
    """

    user_id: typing_extensions.Annotated[EntityIdBase, FieldMetadata(alias="userId")] = pydantic.Field()
    """
    ID that uniquely identifies this app user
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .precondition_base import PreconditionBase
import typing_extensions
from ...core.serialization import FieldMetadata
import pydantic
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ConversationExecutedActionPrecondition(PreconditionBase):
    action_id: typing_extensions.Annotated[str, FieldMetadata(alias="actionId")] = pydantic.Field()
    """
    ID of an action that must have executed in this conversation for the precondition to be met
    """

    app_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="appId")] = pydantic.Field(
        default=None
    )
    """
    App ID that the given actionId belongs to. If not provided, the calling appId will be used.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
import typing_extensions
from ...core.serialization import FieldMetadata
from .action_form_field import ActionFormField
from .chart_spec_schema import ChartSpecSchema


class BotResponse_Text(UniversalBaseModel):
    type: typing.Literal["text"] = "text"
    text: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class BotResponse_ActionForm(UniversalBaseModel):
    type: typing.Literal["actionForm"] = "actionForm"
    id: str
    form_label: typing_extensions.Annotated[str, FieldMetadata(alias="formLabel")]
    fields: typing.List[ActionFormField]
    submit_label: typing_extensions.Annotated[str, FieldMetadata(alias="submitLabel")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class BotResponse_Chart(UniversalBaseModel):
    type: typing.Literal["chart"] = "chart"
    label: str
    spec_schema: typing_extensions.Annotated[ChartSpecSchema, FieldMetadata(alias="specSchema")]
    spec: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


BotResponse = typing.Union[BotResponse_Text, BotResponse_ActionForm, BotResponse_Chart]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ActionEnumOption(UniversalBaseModel):
    label: str = pydantic.Field()
    """
    Will be shown during form display as the user facing string in a dropdown or radio.
    """

    value: typing.Optional[typing.Any] = pydantic.Field(default=None)
    """
    A valid value for the field.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing_extensions
from ...core.serialization import FieldMetadata
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class TranslationRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.translations import TranslationRequest

    TranslationRequest(
        text="Hello world",
        target_language="es",
    )
    """

    text: str = pydantic.Field()
    """
    The text to translate
    """

    target_language: typing_extensions.Annotated[str, FieldMetadata(alias="targetLanguage")] = pydantic.Field()
    """
    The target language to translate to, in ISO 639-1 code format.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    ActionBase,
    ActionEnumOption,
    ActionFormField,
    ActionParameter,
    ActionParameterType,
    ActionResponse,
    AppUser,
    AppUserIdentifier,
    AppUserIdentifyingPropertyType,
    AppUserRequest,
    AppUserResponse,
    BotActionFormResponse,
    BotChartResponse,
    BotConversationMessageType,
    BotMessage,
    BotResponse,
    BotResponseMetadata,
    BotResponse_ActionForm,
    BotResponse_Chart,
    BotResponse_Text,
    BotTextResponse,
    Capability,
    ChartSpecSchema,
    ConversationAnalysis,
    ConversationBase,
    ConversationExecutedActionPrecondition,
    ConversationMessageBase,
    ConversationMessageResponse,
    ConversationMessageResponse_Bot,
    ConversationMessageResponse_User,
    ConversationPrecondition,
    ConversationPrecondition_ActionExecuted,
    ConversationPrecondition_Metadata,
    ConversationPrecondition_Tags,
    ConversationResponse,
    EntityId,
    EntityIdBase,
    EntityType,
    ErrorMessage,
    EventTriggerBase,
    EventTriggerResponse,
    EventTriggerType,
    Feedback,
    FeedbackBase,
    FeedbackType,
    MetadataPrecondition,
    Precondition,
    PreconditionBase,
    PreconditionGroup,
    PreconditionGroupOperator,
    PreconditionOperator,
    Precondition_Conversation,
    Precondition_Group,
    Precondition_User,
    ResponseConfig,
    ResponseLength,
    Sentiment,
    Source,
    TagsPrecondition,
    UserConversationMessageType,
    UserData,
    UserMessage,
    UserMessageAttachment,
    UserMessageBase,
    VisibilityType,
)
from .errors import BadRequestError, NotFoundError, ServerError

__all__ = [
    "ActionBase",
    "ActionEnumOption",
    "ActionFormField",
    "ActionParameter",
    "ActionParameterType",
    "ActionResponse",
    "AppUser",
    "AppUserIdentifier",
    "AppUserIdentifyingPropertyType",
    "AppUserRequest",
    "AppUserResponse",
    "BadRequestError",
    "BotActionFormResponse",
    "BotChartResponse",
    "BotConversationMessageType",
    "BotMessage",
    "BotResponse",
    "BotResponseMetadata",
    "BotResponse_ActionForm",
    "BotResponse_Chart",
    "BotResponse_Text",
    "BotTextResponse",
    "Capability",
    "ChartSpecSchema",
    "ConversationAnalysis",
    "ConversationBase",
    "ConversationExecutedActionPrecondition",
    "ConversationMessageBase",
    "ConversationMessageResponse",
    "ConversationMessageResponse_Bot",
    "ConversationMessageResponse_User",
    "ConversationPrecondition",
    "ConversationPrecondition_ActionExecuted",
    "ConversationPrecondition_Metadata",
    "ConversationPrecondition_Tags",
    "ConversationResponse",
    "EntityId",
    "EntityIdBase",
    "EntityType",
    "ErrorMessage",
    "EventTriggerBase",
    "EventTriggerResponse",
    "EventTriggerType",
    "Feedback",
    "FeedbackBase",
    "FeedbackType",
    "MetadataPrecondition",
    "NotFoundError",
    "Precondition",
    "PreconditionBase",
    "PreconditionGroup",
    "PreconditionGroupOperator",
    "PreconditionOperator",
    "Precondition_Conversation",
    "Precondition_Group",
    "Precondition_User",
    "ResponseConfig",
    "ResponseLength",
    "Sentiment",
    "ServerError",
    "Source",
    "TagsPrecondition",
    "UserConversationMessageType",
    "UserData",
    "UserMessage",
    "UserMessageAttachment",
    "UserMessageBase",
    "VisibilityType",
]



# This file was auto-generated by Fern from our API Definition.

from .event_trigger_base import EventTriggerBase
import typing_extensions
from .entity_id import EntityId
from ...core.serialization import FieldMetadata
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class EventTriggerResponse(EventTriggerBase):
    """
    Examples
    --------
    from mavenagi.commons import EntityId, EventTriggerResponse

    EventTriggerResponse(
        trigger_id=EntityId(
            reference_id="store-in-snowflake",
            app_id="snowflake",
            organization_id="acme",
            agent_id="support",
            type="EVENT_TRIGGER",
        ),
        description="Stores conversation data in Snowflake",
        type="CONVERSATION_CREATED",
    )
    """

    trigger_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="triggerId")] = pydantic.Field()
    """
    ID that uniquely identifies this event trigger
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
